import importlib.util
import os
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import toml


def send_gmail(results, key):
    current_dir = os.path.dirname(__file__)
    package_root = os.path.abspath(os.path.join(current_dir, "..", ".."))

    config_path = os.path.join(package_root, "config.toml")
    encrypt_path = os.path.join(package_root, "manager", "encrypt.py")

    spec = importlib.util.spec_from_file_location("decrypt", encrypt_path)
    decrypt_module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(decrypt_module)

    with open(config_path, "r") as file:
        details = toml.load(file)

    mail_id = details["mail"]["mail_id"]
    encrypted_password = details["mail"]["password"]
    pid = results["pid"]
    error = results["error"]

    password = decrypt_module.decrypt(encrypted_password, key)

    if mail_id.endswith("@gmail.com"):

        message = f"""
                        <html>
                        <body>
                        <p><b>ntfyme :: Diagnostics </b></p>
                        <p><b>Pid:</b> {pid}</p>
                        <p><b>command run:</b> {results['command']}</p>
                        <p><b>Time taken:</b> {results['time_taken']} seconds</p>
                        </body>
                        </html>
                        """
        message1 = f"""
                            <html>
                            <body>
                            <p><b>Output:</b> {results["output"]}</p>
                            <p><b>Return Code:</b> {results["return_code"]}</p>
                            <p><b>Remarks:</b> {results["remarks"]}</p>
                            <p><b>Error:</b> {results["error"]}</p>
                            </body>
                            </html>
                            """
        filename = f"output{pid}.txt"
        message_mail = f"""
        Output:\n{results["output"]}\n
        Error:\n{results["error"]}
        """
        if len(results["output"]) > 1000:
            with open(filename, "w") as f:
                f.write(message_mail)
            with open(filename) as f:
                attachment = MIMEText(f.read())
                attachment.add_header(
                    "Content-Disposition", "attachment", filename=filename
                )
        else:
            message = message + message1

        msg = MIMEMultipart()
        msg["From"] = msg["To"] = mail_id
        success_sub = details["ntfyme"]["success_subject"]
        error_sub = details["ntfyme"]["error_subject"]
        msg["Subject"] = (
            f"ntfyme :: {success_sub}" if error == "none" else f"ntfyme :: {error_sub}"
        )

        msg.attach(MIMEText(message, "html"))
        if len(results["output"]) > 1000:
            msg.attach(attachment)
        server = smtplib.SMTP("smtp.gmail.com: 587")
        server.starttls()
        server.login(msg["From"], password)
        server.sendmail(msg["From"], msg["To"], msg.as_string())
        server.quit()

        # remove the file after sending the mail
        if os.path.exists(filename):
            os.remove(filename)



from plyer import notification


def generate_notif(title, message, timeout):
    """
    General notifier.
    """
    try:
        notification.notify(title=title, message=message, app_icon="", timeout=timeout)
        return 0

    except Exception as e:
        print(f"Error in General Notification as {e}")
        return 1


def term_print(results):
    """
    This function is called only when the terminal_print is set to on in the config file.
    Thus the output will already be printed in the terminal.
    """
    print("\033[92m\nntfyme :: Diagnostics\033[0m")
    print("\033[93mPID:\033[0m", results["pid"])
    print("\033[93mCommand run:\033[0m", results["command"])
    print("\033[93mReturn Code:\033[0m", results["return_code"])
    print("\033[93mRemarks:\033[0m", results["remarks"])
    print("\033[93mTime taken:\033[0m", results["time_taken"], "seconds")
    print("\033[91mError:\n\033[0m"+results["error"])

    return 0



import base64
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


def encrypt(password: str, key: str) -> str:
    # Generate a random salt
    salt = os.urandom(16)

    # Derive a 32-byte encryption key from the provided key using PBKDF2
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=default_backend(),
    )
    encryption_key = kdf.derive(key.encode())

    # Generate a random initialization vector (IV)
    iv = os.urandom(16)

    # Create a Cipher object using the encryption key and IV
    cipher = Cipher(
        algorithms.AES(encryption_key), modes.CFB(iv), backend=default_backend()
    )
    encryptor = cipher.encryptor()

    # Encrypt the password
    encrypted_password = encryptor.update(password.encode()) + encryptor.finalize()

    # Encode the salt, IV, and encrypted password in base64 and concatenate them
    encoded_encrypted_password = base64.b64encode(
        salt + iv + encrypted_password
    ).decode("utf-8")

    return encoded_encrypted_password


def decrypt(encrypted_password: str, key: str) -> str:
    # Decode the base64 encoded string
    encrypted_password_bytes = base64.b64decode(encrypted_password)

    # Extract the salt, IV, and encrypted password
    salt = encrypted_password_bytes[:16]
    iv = encrypted_password_bytes[16:32]
    encrypted_password = encrypted_password_bytes[32:]

    # Derive the encryption key using the same parameters
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=default_backend(),
    )
    encryption_key = kdf.derive(key.encode())

    # Create a Cipher object using the encryption key and IV
    cipher = Cipher(
        algorithms.AES(encryption_key), modes.CFB(iv), backend=default_backend()
    )
    decryptor = cipher.decryptor()

    # Decrypt the password
    decrypted_password = decryptor.update(encrypted_password) + decryptor.finalize()

    return decrypted_password.decode("utf-8")



import subprocess
import sys
import time

from .live_capture import *


def seconds_to_time(seconds):
    """
    Converts seconds to human readable time format.
    """
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    if h == 0:
        if m == 0:
            return f"{s}"
        return f"{m} minutes {s}"
    return f"{h} hours {m} minutes {s}"


def direct_exec(cmd, terminal_print, track_process):

    if cmd is None:
        print("Error: No input provided through direct execution.")
        sys.exit(1)

    start_time = time.time()
    if terminal_print == "on":
        results = capture(cmd, track_process)
    else:
        process = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
        )
        output, error = process.communicate()
        results = {
            "output": output,
            "error": error,
            "pid": process.pid,
            "return_code": process.returncode,
        }
    output = results["output"]
    error = results["error"]
    pid = results["pid"]
    return_code = results["return_code"]

    end_time = time.time()
    time_taken = end_time - start_time
    time_taken = seconds_to_time(time_taken)
    return {
        "output": output,
        "command": "".join(cmd),
        "time_taken": time_taken,
        "pid": pid,
        "error": error if return_code!= 0 else "none",
        "return_code": return_code,
        "remarks": remarks(return_code),
    }



import os
import subprocess

import toml


def macos_config():
    current_dir = os.path.dirname(__file__)
    package_root = os.path.abspath(os.path.join(current_dir, "..", ".."))
    config_path = os.path.join(package_root, "config.toml")

    with open(config_path, "r") as f:
        conf = toml.load(f)

    enabled = conf["local"]["enabled"]
    success_sub = conf["ntfyme"]["success_subject"]
    error_sub = conf["ntfyme"]["error_subject"]
    success_sound = conf["local_macos"]["success_sound"]
    error_sound = conf["local_macos"]["error_sound"]

    return {
        "enabled": enabled,
        "success_sub": success_sub,
        "error_sub": error_sub,
        "success_sound": success_sound,
        "error_sound": error_sound,
    }


def notify_macos(results):
    """
    macos uses osascript for notifications. This command will be used for the notifications.
    """
    configs = macos_config()

    if not configs["enabled"] == "on":
        return 0

    success_sub = configs["success_sub"]
    error_sub = configs["error_sub"]
    success_sound = configs["success_sound"]
    error_sound = configs["error_sound"]
    pid = results["pid"]
    error = results["error"]

    title, message, sound = "", "", ""
    if error != "none":
        title = f"ntfyme :: {error_sub}"
        message = f"Process {pid} ended with failure."
        sound = error_sound
    else:
        title = f"ntfyme :: {success_sub}"
        message = f"Process {pid} ended successfully."
        sound = success_sound
    # AppleScript for notification
    script = f"""
    osascript -e 'display notification "{message}" with title "{title}" sound name "{sound}"'
    """

    try:
        subprocess.run(script, shell=True, check=True)
    except Exception as e:
        print(e)
        return 1



import datetime
import os
import platform

def log_print(log_dict):
    """
    The log file prints prints the formatted log file.
    """
    log_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    pid = log_dict["pid"]
    time_taken = log_dict["time_taken"]
    command = log_dict["command"]
    track_process = log_dict["track_process"]
    return_code = log_dict["return_code"]
    remarks = log_dict["remarks"]

    system = platform.system()
    if system not in ["Windows", "Linux"]:
        system = "Macos"

    log_dir = os.path.dirname(__file__)
    log_file = os.path.join(log_dir, "ntfyme.log")

    with open(log_file, "a") as lg:  # Note: Changed mode to "a" for appending
        status = "Successful" if return_code == 0 else "Error"
        lg.write(f"{log_time} :: PID: {pid} :: Status: {status}\n")
        lg.write(
            f"Command run: {command}\nTime taken: {time_taken} seconds :: OS: {system}\n"
        )
        lg.write(
            f"Key: {log_dict['key']} :: Execution{log_dict['execution']} :: Notify: {log_dict['notify']} :: Track-Process: {track_process}\n"
        )
        lg.write(f"Return Code: {str(return_code)} :: Remarks: {remarks}\n")
        lg.write("\n")


def log_add(results: dict, log_info: dict):
    """
    The log file adds log attribs to the log file
    The output include -
    * results dict
    * debug elements: log_info dict
    """
    log_dict = {
        "pid": results["pid"],
        "command": results["command"],
        "time_taken": results["time_taken"],
        "return_code": results["return_code"],
        "remarks": results["remarks"],
        "key": log_info["key"],
        "execution": log_info["execution"],
        "notify": log_info["notify"],
        "track_process": log_info["track_process"],
    }

    log_print(log_dict)



import os

import toml

from .gen_notif import generate_notif


def windows_config():
    current_dir = os.path.dirname(__file__)
    package_root = os.path.abspath(os.path.join(current_dir, "..", ".."))
    config_path = os.path.join(package_root, "config.toml")

    with open(config_path, "r") as f:
        conf = toml.load(f)
    enabled = conf["local"]["enabled"]
    success_sub = conf["ntfyme"]["success_subject"]
    error_sub = conf["ntfyme"]["error_subject"]
    timeout = conf["local_windows"]["timeout"]
    return {
        "enabled": enabled,
        "success_sub": success_sub,
        "error_sub": error_sub,
        "timeout": timeout,
    }


def notify_windows(results):
    """
    windows uses notify-send as default tool. This will be used for linux local notifications
    """
    configs = windows_config()

    if not configs["enabled"] == "on":
        return 0

    success_sub = configs["success_sub"]
    error_sub = configs["error_sub"]
    timeout = configs["timeout"]
    pid = results["pid"]
    error = results["error"]
    title, message = "", ""
    if error == "none":
        title = f"ntfyme :: {success_sub}"
        message = f"Process {pid} has ended successfully."
    else:
        title = f"ntfyme :: {error_sub}"
        message = f"Process {pid} ended with a failure."

    err_code = generate_notif(title, message, timeout)
    return err_code



import os
import platform

import toml

from ntfyme.utils.local_notify.gen_notif import term_print
from ntfyme.utils.local_notify.linux import notify_linux
from ntfyme.utils.local_notify.macos import notify_macos
from ntfyme.utils.local_notify.windows import notify_windows
from ntfyme.utils.mail.gmail import send_gmail
from ntfyme.utils.mail.telegram import send_telegram


def get_os():
    if platform.system().lower() == "windows":
        return "windows"
    elif platform.system().lower() not in ["linux", "windows"]:
        return "macos"

    else:
        # Check if running under WSL
        with open("/proc/version", "r") as f:
            proc_version = f.read()

        if "microsoft" in proc_version.lower() or "wsl" in proc_version.lower():
            return "wsl"
        return "linux"


def notify_runner(notify_function, name, results, key):
    try:
        if name == "gmail":
            notify_function(results, key)
        else:
            notify_function(results)
        return 1
    except Exception as e:
        print(f"Error occurred in {name} notifier. Error: {e}")
        return 0


def notify(results, key):
    """
    notify_macos   General notification function, which controls the calling of all the notification types
    Local - Linux, Macos
    Remote - mail, telegram
    """
    system = get_os()

    try:
        if system == "linux":
            notify_linux(results)
        elif system == "macos":
            notify_macos(results)
        else:
            notify_windows(results)

    except Exception as e:
        print("Error occurred in notification as", e)

    script_dir = os.path.dirname(os.path.abspath(__file__))
    config_path = os.path.join(script_dir, "config.toml")
    with open(config_path, "r") as file:
        config = toml.load(file)

    # Local terminal printing.
    terminal_print = config["ntfyme"]["terminal_print"]
    if terminal_print == "on":
        term_print(results)

    gmail_output, telegram_output = 0, 0
    if config["mail"]["enabled"] == "on":
        gmail_output = notify_runner(send_gmail, "gmail", results, key)
    if config["telegram"]["enabled"] == "on":
        telegram_output = notify_runner(send_telegram, "telegram", results, key)

    return gmail_output, telegram_output



import importlib.util
import os
import re

import requests
import tomlkit

config_path = os.path.join(os.path.dirname(__file__), "..", "config.toml")


def loop_on_off(choice):
    """
    Loop to check if the input is on or off.
    """
    while choice.lower() not in ["on", "off"]:
        print("\033[91m" + "Invalid choice. Please enter a valid choice." + "\033[0m")
        choice = input("\033[93m" + "Enter your choice: " + "\033[0m")
    return choice


def valid_telegram_chat_id():
    chat_id = ""
    while True:
        chat_id = input("\033[93m" + "Enter Telegram Chat ID: " + "\033[0m")
        if chat_id.isdigit():  # Ensuring the chat ID is an integer
            return int(chat_id)
        print("\033[91m" + "Invalid Telegram Chat ID: Must be an integer." + "\033[0m")


def valid_telegram_token():
    token = ""
    while True:
        token = input("\033[93m" + "Enter Telegram Bot Token: " + "\033[0m")
        response = requests.get(
            f"https://api.telegram.org/bot{token}/getMe", timeout=10
        )
        if response.status_code == 200 and response.json().get("ok"):
            return token
        if response.status_code >= 400:
            print(
                "Unable to authenticate with Telegram API. Please check your internet or input a valid token."
            )
        else:
            print(
                "\033[91m"
                + "Invalid Telegram Bot Token: Must be a valid token."
                + "\033[0m"
            )


def valid_gmail_email():
    gmail_pattern = re.compile(r"^[a-zA-Z0-9_.+-]+@gmail\.com$")
    email = ""
    input_msg = "\033[93m" + "Enter Gmail Email ID: " + "\033[0m"
    while True:
        email = input(input_msg)
        if gmail_pattern.match(email):
            return email
        print(
            "\033[91m" + "Invalid Gmail Email ID: Must be a valid Gmail ID." + "\033[0m"
        )


def telegram_setup():
    """
    Setup for telegram bot.
    """
    print(
        "\n\033[92m"
        + "Welcome to the telegram setup. Before starting this setup, please make sure to have read the guidelines on setting up the telegram bot for ntfyme at: https://github.com/AnirudhG07/ntfyme/blob/main/docs/setup_guide/telegram_bot.md. You will be required to enter your telegram chat_id and token."
        + "\033[0m"
    )
    print("Note: This will not be stored with encryption. ")
    chat_id = valid_telegram_chat_id()
    token = valid_telegram_token()

    enabled = input(
        "\033[93m"
        + "Do you want to enable telegram notifications? (on/off): "
        + "\033[0m"
    )
    enabled = loop_on_off(enabled)

    with open(config_path, "r") as file:
        tele_toml = tomlkit.parse(file.read())
    # Directly modify the document object
    if "telegram" not in tele_toml:
        tele_toml["telegram"] = tomlkit.table()
    tele_toml["telegram"]["chat_id"] = chat_id
    tele_toml["telegram"]["token"] = token
    tele_toml["telegram"]["enabled"] = enabled

    # Write the modified document object back to the file
    with open(config_path, "w") as file:
        file.write(tomlkit.dumps(tele_toml))

    print("Telegram setup complete.")
    return 0


def gmail_setup():
    """
    Setup for gmail.
    """
    # color it green
    print(
        "\n\033[92m"
        + "Welcome to the gmail setup. Before starting this setup, please make sure to have read the guidelines on setting up your gmail for security purposes. You will be required to enter your gmail id, password and ntfyme_key."
        + "\033[0m"
    )
    print("Note: This will be stored with encryption for security purposes.")
    mail_id = valid_gmail_email()
    password = input("\033[93m" + "Enter your gmail app password: " + "\033[0m")
    key = input("\033[93m" + "Enter your ntfyme_key: " + "\033[0m")
    enabled = input(
        "\033[93m"
        + "Do you want to enable gmail notification? Enabling this will ask for your ntfyme_key everytime you run ntfyme. (on/off): "
        + "\033[0m"
    )
    enabled = loop_on_off(enabled)

    encrypt_path = os.path.join(os.path.dirname(__file__), "encrypt.py")
    spec = importlib.util.spec_from_file_location("encrypt", encrypt_path)
    encrypt_modeule = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(encrypt_modeule)
    encrypted_password = encrypt_modeule.encrypt(password, key)

    with open(config_path, "r") as file:
        gmail_toml = tomlkit.parse(file.read())
    # Directly modify the document object
    if "telegram" not in gmail_toml:
        gmail_toml["telegram"] = tomlkit.table()

    gmail_toml["mail"]["mail_id"] = mail_id
    gmail_toml["mail"]["password"] = encrypted_password
    gmail_toml["mail"]["enabled"] = enabled
    # Write the modified document object back to the file
    with open(config_path, "w") as file:
        file.write(tomlkit.dumps(gmail_toml))

    print("Gmail setup complete.")
    return 0


def setup():
    """
    Main interaction function for inputting keys and password and writing to config.toml
    """
    print(
        "\033[92m"
        + "Welcome to the ntfyme setup. This setup will help you configure your notification settings.\n"
        + "\033[0m"
        + "\033[95m"
        + "Note: If this is not your first time configuring ntfyme, the previous settings will be overwritten."
        + "\033[0m\n",
    )
    # color option purple and bold and text yello
    print("\033[91m" + "[1]" + "\033[0m" + "\033[94m" + " Gmail setup" + "\033[0m")
    print("\033[91m" + "[2]" + "\033[0m" + "\033[94m" + " Telegram setup" + "\033[0m")
    print("\033[91m" + "[3]" + "\033[0m" + "\033[94m" + " Exit" + "\033[0m")

    choice = input("\033[93m" + "\nEnter your choice: " + "\033[0m")
    while choice not in ["1", "2", "3"]:
        print("Invalid choice. Please enter a valid choice.")
        choice = input("Enter your choice: ")

    if choice == "1":
        gmail_setup()
    elif choice == "2":
        telegram_setup()
    elif choice == "3":
        return 0



import os

import requests
import toml

def send_telegram(results):

    current_dir = os.path.dirname(__file__)
    package_root = os.path.abspath(os.path.join(current_dir, "..", ".."))
    config_path = os.path.join(package_root, "config.toml")

    with open(config_path, "r") as file:
        details = toml.load(file)

    chat_id =   details["telegram"]["chat_id"]
    token = details["telegram"]["token"]
    url = f"https://api.telegram.org/bot{token}/sendMessage"
    
    error = results["error"]
    pid = results["pid"]

    message = f"*PID:* {pid}\n*Command run:* {results['command']}\n*Time taken:* {results['time_taken']} seconds\n"
    message_telegram = f"*Output*:\n{results['output']}\n*Return Code*:\n{results['return_code']}\n*Remarks*:\n{results['remarks']}\n*Error:* {error}"

    filename = f"output{pid}.txt"
    if len(results["output"]) > 1000:
        with open(filename, "w") as file:
            file.write(message_telegram)
        with open(filename, "r") as file:
            files = {'document': (filename, file)}
            data = {"chat_id": chat_id, "text": message, "parse_mode": "Markdown"}
            requests.post(url, json=data)
            requests.post(f"https://api.telegram.org/bot{token}/sendDocument", data={"chat_id": chat_id}, files=files)
            
    else:
        message = message + message_telegram
        data = {"chat_id": chat_id, "text": message, "parse_mode": "Markdown"}
        requests.post(url, json=data)

    if os.path.exists(filename):
        os.remove(filename)



import os
import platform
import subprocess
from argparse import ArgumentParser, RawTextHelpFormatter

import toml

from ntfyme.cmd.cmd_direct import direct_exec
from ntfyme.cmd.cmd_pipe import pipe_exec
from ntfyme.manager.encrypt import encrypt
from ntfyme.manager.setup_interaction import setup
from ntfyme.notification import notify
from ntfyme.utils.log.log import log_add


def main():
    """
    -> Handles the flags and errors
    -> calls functions based on the flags

    Arguments:
        None: Assumes the user wants to run the main command through pipe
        --cmd or -c : Input the command to cli as 'ntfyme --cmd <command>'
        --enc or -e : Encrypt password with your key for safety
        --log : The command log of ntfyme
        --config: The configuration file of ntfyme

        --help or -h : Shows the help message
        --version or -v : Shows the version of the program
    """

    parser = ArgumentParser(description="ntfyme")
    parser = ArgumentParser(
        description="""ntfyme is a simple notification tool to notify yourself when a long running process ends with local ping, gmail, telegram, etc. For setup guidelines or if you are facing any issue, checkout the official github repository at: https://github.com/AnirudhG07/ntfyme.""",
        formatter_class=RawTextHelpFormatter,
    )
    parser.add_argument("--version", "-v", action="version", version="ntfyme v0.0.1")
    parser.add_argument("--cmd", "-c", help="Run the command through direct execution")
    parser.add_argument(
        "--log", "-l", action="store_true", help="The command log of ntfyme"
    )
    parser.add_argument(
        "--config", action="store_true", help="The configuration file of ntfyme"
    )
    parser.add_argument(
        "--enc",
        "-e",
        action="store_true",
        help="Encrypting password through ntfyme_key",
    )
    parser.add_argument(
        "--interactive-setup",
        "-i",
        action="store_true",
        help="Interactively setup your notification configuration",
    )
    parser.add_argument(
        "--track-process",
        "-t",
        action="store_true",
        help="Track the process for suspensions and terminate if stalled for a long time",
    )

    args = parser.parse_args()
    script_dir = os.path.dirname(os.path.abspath(__file__))
    config_path = os.path.join(script_dir, "config.toml")
    with open(config_path, "r") as file:
        config = toml.load(file)

    log_pager = config["ntfyme"]["log_pager"]
    terminal_print = config["ntfyme"]["terminal_print"]

    # Handling log and config arguments
    if args.log:
        log_path = os.path.join(script_dir, "utils", "log", "ntfyme.log")
        try:
            subprocess.run([log_pager, log_path])
        except Exception as e:
            print(f"Error occurred in opening log file. Error: {e}")
        return 0

    if args.config:
        config_path = os.path.join(script_dir, "config.toml")
        editor = os.getenv("EDITOR", "nano")  # Default to nano if EDITOR is not set
        # Open the config.toml file in the editor
        if platform.system() != "Windows":
            print(
                "For security reasons, please provide your sudo password to edit the config file."
            )
            subprocess.run(["sudo", editor, config_path])
            return 0
        subprocess.run([editor, config_path])
        return 0

    if args.enc:
        print(
            "Please provide your ntfyme_key for encrypting your password. This key is same throughout ntfyme. Whataver output you get will be based on the same key, please be mindful of the usage."
        )
        key = input("Enter your ntfyme_key: ")
        password = input("Enter your password: ")
        encrypted_password = encrypt(password, key)
        print(f"Encrypted password: {encrypted_password}")
        return 0

    if args.interactive_setup:
        setup()
        return 0

    results, key = None, None
    log_info = {}

    log_info["key"]=0
    if config["mail"]["enabled"] == "on":
        key = input("Enter your ntfyme_key: ")
        if not key:
            log_info["key"] = "1"


    if args.track_process:
        track_process = "on"
        log_info["track_process"] = "0"
    else:
        track_process = "off"
        log_info["track_process"] = "1"

    try:
        if args.cmd:
            results = direct_exec(args.cmd, terminal_print, track_process)
            log_info["execution"] = ": Direct :: 0"
        else:
            results = pipe_exec(terminal_print, track_process)
            log_info["execution"] = ": Pipe :: 0"

    except Exception as e:
        print(f"Error occurred in command execution. Error: {e}")
        log_info["error"] = "Execution: 1"

    try:
        notify(results, key)
        log_info["notify"] = "0"
    except Exception as e:
        print(f"Error occurred in notification. Error: {e}")
        log_info["notify"] = "1"

    log_add(results, log_info)
    return 0


if __name__ == "__main__":
    main()



import gzip
import os
import shutil
import sys

from setuptools import find_packages, setup

description = """
Have you ever wanted a simple tool in your terminal which gives you notification when the program is done, like some computer notification or some mail.
This tool helps you ease your life with appropriate notifications you can set in your OS, get mail regarding it and best of all, it is VERY SIMPLE TO USE!

## Features

- Very easy to use and setup.
- Cross platform support for Windows, MacOS and Linux.
- Get local notifications when your command is done.
- Get notifications like gmail, telegram bot, etc. when your command is done.
- Get information about output, errors, time taken, pid and more in the notification.
- Track your process for possible suspensions and get notified after thorough diagnostics.

## Usages
There are mainly two ways to use the tool to get a notification -

### 1. Direct Run

For "direct running", you can follow the below format -

```bash
ntfyme -c <command>
```

### 2. Piping the command

For "piping the command", you can follow the below format -

```bash
echo <command> | ntfyme
```

For more information, visit the official github repository https://github.com/AnirudhG07/ntfyme
"""

setup(
    name="ntfyme",
    version="0.0.1",
    description="Simple notification tool to notify yourself when a long running process ends with local ping, gmail, telegram, etc.",
    long_description=description,
    long_description_content_type="text/markdown",
    url="https://github.com/AnirudhG07/ntfyme",
    author="Anirudh Gupta",
    package_data={
        "ntfyme": ["config.toml", "utils/*, manager/*, cmd/*"],
    },
    packages=find_packages(),
    install_requires=[
        "toml",
        "plyer",
        "tomlkit",
        "cryptography",
        "requests",
    ],
    keywords=["notification", "notify", "cli", "python"],
    python_requires=">=3.10",
    entry_points={
        "console_scripts": [
            "ntfyme=ntfyme.main:main",
        ],
    },
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
    ],
)



import os
import subprocess
import sys
import threading
import time

import toml

def remarks(return_code:int)->str:
    """
    Gives remarks based on the return code
    """
    return_code_remarks = {
    0: "Success. The command executed successfully without any errors.",
    1: "General error. The command failed due to a general error.",
    2: "Misuse of shell built-ins (according to Bash documentation).",
    126: "Command invoked cannot execute. This might happen if the command is not executable.",
    127: "Command not found. This indicates that the command was not found in the system's PATH.",
    128: "Invalid argument to exit. This indicates that the exit status is out of the range of valid exit statuses.",
    130: "Script terminated by Control-C. This indicates that the script was terminated by the user pressing Control-C.",
    137: "Command terminated by signal 9 (SIGKILL)."
    }
    return return_code_remarks.get(return_code, "Unknown return code")

class ProcessMonitor:
    def __init__(self, timeout, iterations):
        self.timeout = timeout
        self.iterations = iterations
        self.last_activity = time.time()
        self.lock = threading.Lock()
        self.stalled_count = 0

    def update_activity(self):
        with self.lock:
            self.last_activity = time.time()
            self.stalled_count = 0  # Reset stall count on activity

    def check_timeout(self):
        with self.lock:
            return time.time() - self.last_activity > self.timeout

    def increment_stalled_count(self):
        with self.lock:
            self.stalled_count += 1
            return self.stalled_count


def read_output(pipe, output_list, print_func, monitor=None):
    """Read lines or characters from a pipe and print them, storing them in a list."""
    while True:
        data = pipe.read(1)  # Read one character at a time
        if not data:
            break
        output_list.append(data)
        print_func(data, end="", flush=True)
        if monitor:
            monitor.update_activity()


def monitor_stall(monitor, process, check_interval):
    """Monitor for stalling and log an alert if detected."""
    while process.poll() is None:  # Continue while the process is running
        if monitor.check_timeout():
            stalled_count = monitor.increment_stalled_count()
            print(
                "Warning: Process may be stalled. No output for a while.",
                file=sys.stderr,
            )
            if stalled_count >= monitor.iterations:
                print("Error: Process output is stalled.", file=sys.stderr)
                process.terminate()  # Terminate the process
                break
        time.sleep(check_interval)  # Check every sleep_time seconds


def capture(cmd, track_process):

    config_path = os.path.join(os.path.dirname(__file__), "..", "config.toml")
    with open(config_path, "r") as f:
        config = toml.load(f)

    timeout = config["suspend"]["timeout"]
    iterations = config["suspend"]["iterations"]
    enabled = True if track_process == "on" else False
    check_interval = config["suspend"]["check_interval"]

    # Set the PYTHONUNBUFFERED environment variable to force unbuffered output
    env = os.environ.copy()
    env["PYTHONUNBUFFERED"] = "1"

    monitor = ProcessMonitor(timeout, iterations) if enabled else None

    process = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
        text=True,
        bufsize=0,
        env=env,
    )
    output, error = [], []

    stdout_thread = threading.Thread(
        target=read_output, args=(process.stdout, output, print, monitor)
    )
    stderr_thread = threading.Thread(
        target=read_output,
        args=(
            process.stderr,
            error,
            lambda x, **kwargs: None,  # Do nothing instead of printing
            monitor,
        ),
    )

    stdout_thread.start()
    stderr_thread.start()

    if enabled:
        stall_thread = threading.Thread(
            target=monitor_stall, args=(monitor, process, check_interval)
        )
        stall_thread.start()

    stdout_thread.join()
    stderr_thread.join()
    process.wait()

    if enabled:
        stall_thread.join()

    error_message = "".join(error)
    if enabled and monitor.stalled_count >= iterations:
        error_message += "Error: Process output is stalled."


    return {
        "output": "".join(output),
        "error": error_message,
        "pid": process.pid,
        "return_code": process.returncode,
    }



import os
import subprocess

import toml


def linux_config():
    current_dir = os.path.dirname(__file__)
    package_root = os.path.abspath(os.path.join(current_dir, "..", ".."))
    config_path = os.path.join(package_root, "config.toml")

    with open(config_path, "r") as f:
        conf = toml.load(f)
    enabled = conf["local"]["enabled"]
    success_sub = conf["ntfyme"]["success_subject"]
    error_sub = conf["ntfyme"]["error_subject"]

    return {
        "enabled": enabled,
        "success_sub": success_sub,
        "error_sub": error_sub,
    }


def notify_linux(results):
    """
    Linux uses notify-send as default tool. This will be used for linux local notifications
    """
    configs = linux_config()

    if not configs["enabled"] == "on":
        return

    success_sub = configs["success_sub"]
    error_sub = configs["error_sub"]
    pid = results["pid"]
    error = results["error"]

    try:
        if error == "none":
            subprocess.run(
                [
                    "notify-send",
                    f"ntfyme :: {success_sub}",
                    f"Process {pid} has ended successfully.",
                ]
            )
            return 0
        subprocess.run(
            [
                "notify-send",
                f"ntfyme :: {error_sub}",
                f"Process {pid} ended with a failure.",
            ]
        )
        return 0
    except Exception as e:
        subprocess.run(
            [
                "notify-send",
                "ntfyme: Notification error",
                f"Error {e} occurred in processing your notification request",
            ]
        )
        return 1
